==> fio-1.40.git.tar.gz: fio-1.40/lib/rand.c <==
/*
  This is a maximally equidistributed combined Tausworthe generator
  based on code from GNU Scientific Library 1.5 (30 Jun 2004)

   x_n = (s1_n ^ s2_n ^ s3_n)

   s1_{n+1} = (((s1_n & 4294967294) <<12) ^ (((s1_n <<13) ^ s1_n) >>19))
   s2_{n+1} = (((s2_n & 4294967288) << 4) ^ (((s2_n << 2) ^ s2_n) >>25))
   s3_{n+1} = (((s3_n & 4294967280) <<17) ^ (((s3_n << 3) ^ s3_n) >>11))

   The period of this generator is about 2^88.

   From: P. L'Ecuyer, "Maximally Equidistributed Combined Tausworthe
   Generators", Mathematics of Computation, 65, 213 (1996), 203--213.

   This is available on the net from L'Ecuyer's home page,

   http://www.iro.umontreal.ca/~lecuyer/myftp/papers/tausme.ps
   ftp://ftp.iro.umontreal.ca/pub/simulation/lecuyer/papers/tausme.ps

   There is an erratum in the paper "Tables of Maximally
   Equidistributed Combined LFSR Generators", Mathematics of
   Computation, 68, 225 (1999), 261--269:
   http://www.iro.umontreal.ca/~lecuyer/myftp/papers/tausme2.ps

        ... the k_j most significant bits of z_j must be non-
        zero, for each j. (Note: this restriction also applies to the
        computer code given in [4], but was mistakenly not mentioned in
        that paper.)

   This affects the seeding procedure by imposing the requirement
   s1 > 1, s2 > 7, s3 > 15.

*/

#include "rand.h"

struct frand_state __fio_rand_state;

static inline int __seed(unsigned int x, unsigned int m)
{
	return (x < m) ? x + m : x;
}

void init_rand(struct frand_state *state)
{
#define LCG(x)  ((x) * 69069)   /* super-duper LCG */

	state->s1 = __seed(LCG((2^31) + (2^17) + (2^7)), 1);
	state->s2 = __seed(LCG(state->s1), 7);
	state->s3 = __seed(LCG(state->s2), 15);

	__rand(state);
	__rand(state);
	__rand(state);
	__rand(state);
	__rand(state);
	__rand(state);
}

==> fio-1.41.3.git.tar.gz: fio-1.41.3/lib/rand.c <==
/*
  This is a maximally equidistributed combined Tausworthe generator
  based on code from GNU Scientific Library 1.5 (30 Jun 2004)

   x_n = (s1_n ^ s2_n ^ s3_n)

   s1_{n+1} = (((s1_n & 4294967294) <<12) ^ (((s1_n <<13) ^ s1_n) >>19))
   s2_{n+1} = (((s2_n & 4294967288) << 4) ^ (((s2_n << 2) ^ s2_n) >>25))
   s3_{n+1} = (((s3_n & 4294967280) <<17) ^ (((s3_n << 3) ^ s3_n) >>11))

   The period of this generator is about 2^88.

   From: P. L'Ecuyer, "Maximally Equidistributed Combined Tausworthe
   Generators", Mathematics of Computation, 65, 213 (1996), 203--213.

   This is available on the net from L'Ecuyer's home page,

   http://www.iro.umontreal.ca/~lecuyer/myftp/papers/tausme.ps
   ftp://ftp.iro.umontreal.ca/pub/simulation/lecuyer/papers/tausme.ps

   There is an erratum in the paper "Tables of Maximally
   Equidistributed Combined LFSR Generators", Mathematics of
   Computation, 68, 225 (1999), 261--269:
   http://www.iro.umontreal.ca/~lecuyer/myftp/papers/tausme2.ps

        ... the k_j most significant bits of z_j must be non-
        zero, for each j. (Note: this restriction also applies to the
        computer code given in [4], but was mistakenly not mentioned in
        that paper.)

   This affects the seeding procedure by imposing the requirement
   s1 > 1, s2 > 7, s3 > 15.

*/

#include "rand.h"
#include "../hash.h"

struct frand_state __fio_rand_state;

static inline int __seed(unsigned int x, unsigned int m)
{
	return (x < m) ? x + m : x;
}

void init_rand(struct frand_state *state)
{
#define LCG(x)  ((x) * 69069)   /* super-duper LCG */

	state->s1 = __seed(LCG((2^31) + (2^17) + (2^7)), 1);
	state->s2 = __seed(LCG(state->s1), 7);
	state->s3 = __seed(LCG(state->s2), 15);

	__rand(state);
	__rand(state);
	__rand(state);
	__rand(state);
	__rand(state);
	__rand(state);
}

void fill_random_buf(void *buf, unsigned int len)
{
	unsigned long r = __rand(&__fio_rand_state);
	long *ptr = buf;

	if (sizeof(int) != sizeof(*ptr))
		r *= (unsigned long) __rand(&__fio_rand_state);

	while ((void *) ptr - buf < len) {
		*ptr = r;
		ptr++;
		r *= GOLDEN_RATIO_PRIME;
		r >>= 3;
	}
}
